@@ -4,7 +4,7 @@
  * GChemPaint library
  * application.cc 
  *
- * Copyright (C) 2004-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -26,11 +26,13 @@
 #include "application.h"
 #include "document.h"
 #include "electron.h"
+#include "fragment.h"
 #include "text.h"
 #include "plugin.h"
 #include "mesomer.h"
 #include "mesomery.h"
 #include "mesomery-arrow.h"
+#include "molecule.h"
 #include "newfiledlg.h"
 #include "reaction.h"
 #include "reactant.h"
@@ -178,6 +180,16 @@ static void load_globs ()
 namespace gcp {
 
 // Objects creation static methods
+static Object* CreateAtom ()
+{
+	return new Atom ();
+}
+
+static Object* CreateBond ()
+{
+	return new Bond ();
+}
+
 static Object* CreateMolecule ()
 {
 	return new Molecule ();
@@ -239,12 +251,19 @@ bool	Application::m_Have_InChI = false;
 
 // FIXME: Remove for stable versions
 #undef PACKAGE
-#define PACKAGE "gchemutils-unstable" 
+#define PACKAGE "gchemutils-unstable"
 
+#ifdef HAVE_GO_CONF_SYNC
+static void on_config_changed (GOConfNode *node, gchar const *key, Application *app)
+{
+	app->OnConfigChanged (node, key);
+}
+#else
 static void on_config_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, Application *app)
 {
 	app->OnConfigChanged (client, cnxn_id, entry);
 }
+#endif
 
 Application::Application ():
 	gcu::Application ("GChemPaint", DATADIR, "gchempaint-unstable", "gchempaint")
@@ -255,7 +274,6 @@ Application::Application ():
 	m_NumWindow = 1;
 
 	if (!m_bInit) {
-		libgoffice_init ();
 		/* Initialize plugins manager */
 		gcu::Loader::Init ();
 		// Check for programs
@@ -283,6 +301,8 @@ Application::Application ():
 		}
 
 		// Initialize types
+		Object::AddType ("atom", CreateAtom, AtomType);
+		Object::AddType ("bond", CreateBond, gcu::BondType);
 		Object::AddType ("molecule", CreateMolecule, MoleculeType);
 		Object::AddType ("reaction", CreateReaction, ReactionType);
 		Object::SetCreationLabel (ReactionType, _("Create a new reaction"));
@@ -340,7 +360,10 @@ Application::Application ():
 	bool found = Loader::GetFirstLoader (it);
 	while (found) {
 		if ((*it).second.supports2D) {
-			// TODO: use this loader
+			if ((*it).second.read)
+				AddMimeType (m_SupportedMimeTypes, (*it).first);
+			if ((*it).second.write)
+				AddMimeType (m_WriteableMimeTypes, (*it).first);
 		}
 		found = Loader::GetNextLoader (it);
 	}
@@ -349,8 +372,6 @@ Application::Application ():
 	TestSupportedType ("chemical/x-mdl-molfile");
 	TestSupportedType ("chemical/x-pdb");
 	TestSupportedType ("chemical/x-xyz");
-	TestSupportedType ("chemical/x-cdx");
-	TestSupportedType ("chemical/x-cdxml");
 	TestSupportedType ("chemical/x-ncbi-asn1");
 	TestSupportedType ("chemical/x-ncbi-asn1-binary");
 	TestSupportedType ("chemical/x-ncbi-asn1-xml");
@@ -368,15 +389,23 @@ Application::Application ():
 		}
 	}
 	
-	m_ConfClient = gconf_client_get_default ();
-	gconf_client_add_dir (m_ConfClient, "/apps/gchempaint/settings", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-	GError *error = NULL;
-	GCU_GCONF_GET (ROOTDIR"compression", int, CompressionLevel, 0)
-	GCU_GCONF_GET (ROOTDIR"tearable-mendeleiev", bool, TearableMendeleiev, false)
+#ifdef HAVE_GO_CONF_SYNC
+		m_ConfNode = go_conf_get_node (GetConfDir (), GCP_CONF_DIR_SETTINGS);
+#else
+		GError *error = NULL;
+		m_ConfClient = gconf_client_get_default ();
+		gconf_client_add_dir (m_ConfClient, "/apps/gchemutils/paint/settings", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+#endif
+	GCU_GCONF_GET ("compression", int, CompressionLevel, 0)
+	GCU_GCONF_GET ("tearable-mendeleiev", bool, TearableMendeleiev, false)
 	bool CopyAsText;
-	GCU_GCONF_GET (ROOTDIR"copy-as-text", bool, CopyAsText, false)
+	GCU_GCONF_GET ("copy-as-text", bool, CopyAsText, false)
 	ClipboardFormats = CopyAsText? GCP_CLIPBOARD_ALL: GCP_CLIPBOARD_NO_TEXT;
-	m_NotificationId = gconf_client_notify_add (m_ConfClient, "/apps/gchempaint/settings", (GConfClientNotifyFunc) on_config_changed, this, NULL, NULL);
+#ifdef HAVE_GO_CONF_SYNC
+	m_NotificationId = go_conf_add_monitor (m_ConfNode, NULL, (GOConfMonitorFunc) on_config_changed, this);
+#else
+	m_NotificationId = gconf_client_notify_add (m_ConfClient, "/apps/gchemutils/paint/settings", (GConfClientNotifyFunc) on_config_changed, this, NULL, NULL);
+#endif
 	// make themes permanent with this as a dummy client
 	list <string> Names = TheThemeManager.GetThemesNames ();
 	list <string>::iterator j, jend = Names.end ();
@@ -398,6 +427,17 @@ Application::~Application ()
 		xmlFreeDoc (XmlDoc);
 	m_SupportedMimeTypes.clear ();
 	delete m_Dummy;
+#ifdef HAVE_GO_CONF_SYNC
+	go_conf_remove_monitor (m_NotificationId);
+	go_conf_free_node (m_ConfNode);
+	m_ConfNode = NULL;
+#else
+	gconf_client_notify_remove (m_ConfClient, m_NotificationId);
+	gconf_client_remove_dir (m_ConfClient, "/apps/gchemutils/gl", NULL);
+	g_object_unref (m_ConfClient);
+	m_ConfClient = NULL;
+#endif
+	TheThemeManager.Shutdown ();
 }
 
 void Application::ActivateTool (const string& toolname, bool activate)
@@ -545,7 +585,8 @@ bool Application::FileProcess (const gchar* filename, const gchar* mime_type, bo
 				m_pActiveDoc->ExportImage (filename2, pixbuf_type, GetImageResolution ());
 				break;
 			default:
-				if (!strcmp (mime_type, "application/x-gchempaint"))
+				if (Save (filename2, mime_type, pDoc));
+				else if (!strcmp (mime_type, "application/x-gchempaint"))
 					SaveGcp (filename2, pDoc);
 				else
 					SaveWithBabel (filename2, mime_type, pDoc);
@@ -571,10 +612,26 @@ bool Application::FileProcess (const gchar* filename, const gchar* mime_type, bo
 				filename2 = filename;
 			}
 		}
-		if (!strcmp (mime_type, "application/x-gchempaint"))
-			OpenGcp (filename2, pDoc);
-		else
-			OpenWithBabel (filename2, mime_type, pDoc);
+		bool create;
+		if (!pDoc || !pDoc->GetEmpty () || pDoc->GetDirty ()) {
+			create = true;
+			OnFileNew ();
+			pDoc = m_pActiveDoc;
+		}
+		pDoc->SetFileName(filename2, mime_type);
+		if (Load (filename2, mime_type, pDoc)) {
+			pDoc->GetView ()->AddObject (pDoc);
+			pDoc->GetView ()->Update (pDoc);
+		} else {
+			if (create) {
+				pDoc->GetWindow ()->Destroy ();;
+				pDoc =NULL;
+			}
+			if (!strcmp (mime_type, "application/x-gchempaint"))
+				OpenGcp (filename2, pDoc);
+			else
+				OpenWithBabel (filename2, mime_type, pDoc);
+		}
 	}
 	return false;
 }
@@ -1041,30 +1098,47 @@ void Application::ActivateWindowsActionWidget (const char *path, bool activate)
 	}
 }
 
+#ifdef HAVE_GO_CONF_SYNC
+void Application::OnConfigChanged (GOConfNode *node, gchar const *name)
+{
+#else
 void Application::OnConfigChanged (GConfClient *client, guint cnxn_id, GConfEntry *entry)
 {
 	if (client != m_ConfClient)
 		return;	// we might want an error message?
 	if (cnxn_id != m_NotificationId)
 		return;	// we might want an error message?
-	if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"compression")) {
-		CompressionLevel = gconf_value_get_int (gconf_entry_get_value (entry));
-	} else 	if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"tearable-mendeleiev")) {
-		TearableMendeleiev = gconf_value_get_bool (gconf_entry_get_value (entry));
-		Tools *ToolsBox = dynamic_cast<Tools*> (GetDialog ("tools"));
-		if (ToolsBox)
-			ToolsBox->Update ();
-	} else 	if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"copy-as-text"))
-		ClipboardFormats = (gconf_value_get_bool (gconf_entry_get_value (entry)))? GCP_CLIPBOARD_ALL: GCP_CLIPBOARD_NO_TEXT;
+#endif
+	GCU_UPDATE_KEY ("compression", int, CompressionLevel, {})
+	GCU_UPDATE_KEY ("tearable-mendeleiev", bool, TearableMendeleiev,
+					{
+						Tools *ToolsBox = dynamic_cast<Tools*> (GetDialog ("tools"));
+						if (ToolsBox)
+							ToolsBox->Update ();
+					})
+	bool CopyAsText;
+	GCU_UPDATE_KEY ("copy-as-text", bool, CopyAsText, ClipboardFormats = CopyAsText?GCP_CLIPBOARD_ALL: GCP_CLIPBOARD_NO_TEXT;)
+}
+
+void Application::AddMimeType (list<string> &l, string const& mime_type)
+{
+	list<string>::iterator i, iend = l.end ();
+	for (i = l.begin (); i != iend; i++)
+		if (*i == mime_type)
+			break;
+	if (i == iend)
+		l.push_back (mime_type);
+	else
+		g_warning ("Duplicate mime type: %s", mime_type.c_str ());
 }
 
 void Application::TestSupportedType (char const *mime_type)
 {
 	OBFormat *f = OBConversion::FormatFromMIME (mime_type);
 	if (f != NULL) {
-		m_SupportedMimeTypes.push_back (mime_type);
+		AddMimeType (m_SupportedMimeTypes, mime_type);
 		if (!(f->Flags () & NOTWRITABLE))
-			m_WriteableMimeTypes.push_back (mime_type);
+			AddMimeType (m_WriteableMimeTypes, mime_type);
 	}
 }
 
@@ -1115,4 +1189,9 @@ void Application::AddOptions (GOptionContext *context)
 		g_option_context_add_main_entries (context, (*i).entries, (*i).translation_domain);
 }
 
+gcu::Document *Application::CreateNewDocument ()
+{
+	return new Document (this, true, NULL);
+}
+
 }	//	namespace gcp
