@@ -4,7 +4,7 @@
  * GChemPaint library
  * atom.cc
  *
- * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -40,8 +40,12 @@
 #include <openbabel/mol.h>
 #include <glib/gi18n-lib.h>
 #include <cmath>
+#include <cstdlib>
+#include <cstring>
 
 using namespace gcu;
+using namespace OpenBabel;
+using namespace std;
 
 #define ATOM_EPSILON 0.1
 
@@ -67,6 +71,7 @@ Atom::Atom (): gcu::Atom (),
 	m_ChargeAutoPos = true;
 	m_Layout = m_ChargeLayout = NULL;
 	m_DrawCircle = false;
+	m_InkRect.width = m_HInkRect.width = 0.;
 }
 
 Atom::~Atom ()
@@ -108,6 +113,7 @@ Atom::Atom (int Z, double x, double y, double z): gcu::Atom (Z, x, y, z),
 	m_ChargeAutoPos = true;
 	m_Layout = m_ChargeLayout = NULL;
 	m_DrawCircle = false;
+	m_InkRect.width = m_HInkRect.width = 0.;
 }
 
 Atom::Atom (OBAtom* atom): gcu::Atom (),
@@ -135,18 +141,21 @@ Atom::Atom (OBAtom* atom): gcu::Atom (),
 	m_Layout = m_ChargeLayout = NULL;
 	m_DrawCircle = false;
 	m_Charge = atom->GetFormalCharge ();
+	m_InkRect.width = m_HInkRect.width = 0.;
 }
 
 void Atom::SetZ (int Z)
 {
 	gcu::Atom::SetZ (Z);
+	if (Z < 1)
+		return;
 	m_Element = Element::GetElement (m_Z);
-	if ((m_Valence = m_Element->GetDefaultValence ()))
+	if ((m_Valence = (m_Element)? m_Element->GetDefaultValence (): 0))
 		m_HPos = (m_HPosStyle == AUTO_HPOS)? GetBestSide(): m_HPosStyle;
 	else
 		m_nH = 0;
-	int max = m_Element->GetMaxValenceElectrons ();
-	int diff = m_Element->GetTotalValenceElectrons () - m_Element->GetValenceElectrons ();
+	int max = (m_Element)? m_Element->GetMaxValenceElectrons (): 0;
+	int diff = (m_Element)? m_Element->GetTotalValenceElectrons () - m_Element->GetValenceElectrons (): 0;
 	switch (max) {
 	case 2:
 		m_ValenceOrbitals = 1;
@@ -264,14 +273,14 @@ void Atom::Update ()
 	} else {
 		m_nH = 0;
 		if (m_ChargeAuto || !m_Charge) {
-			m_Charge = m_Element->GetValenceElectrons () - 2 * nexplp - nexplu - nbonds;
+			m_Charge = (m_Element)? m_Element->GetValenceElectrons () - 2 * nexplp - nexplu - nbonds: 0;
 			if (m_Charge > 0)
 				m_Charge = 0;
 			m_ChargeAuto = true;
 		}
 	}
 	Document *pDoc = (Document *) GetDocument ();
-	if (pDoc)
+	if (pDoc && pDoc->GetView ())
 		m_Changed = pDoc->GetView ()->GetNbWidgets ();
 	m_AvailPosCached = false;
 	if (nbonds && GetZ () == 6) {
@@ -308,23 +317,16 @@ void Atom::Update ()
 		}
 	}
 }
-	
-bool Atom::IsInCycle (Cycle* pCycle)
-{
-	map<gcu::Atom*, gcu::Bond*>::iterator i, end = m_Bonds.end ();
-	for (i = m_Bonds.begin (); i != end; i++)
-		if (((Bond*) (*i).second)->IsInCycle (pCycle))
-		return true;
-	return false;
-}
 
 void Atom::Add (GtkWidget* w)
 {
-	if (!w)
+	if (!w || !GetZ ())
 		return;
 	if (m_Changed > 0)
 		m_Changed--;
 	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (w), "data");
+	if (pData->Items[this] != NULL)
+		return;
 	View* pView = pData->m_View;
 	Theme *pTheme = pView->GetDoc ()->GetTheme ();
 	if (m_Layout == NULL) {
@@ -587,9 +589,11 @@ void Atom::Add (GtkWidget* w)
 
 void Atom::Update (GtkWidget* w)
 {
-	if (!w)
+	if (!w || !GetZ ())
 		return;
 	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (w), "data");
+	if (pData->Items[this] == NULL)
+		return;
 	Theme *pTheme = pData->m_View->GetDoc ()->GetTheme ();
 	double x, y, xc = 0., yc;
 	GetCoords (&x, &y);
