@@ -26,7 +26,6 @@
 #include "bond.h"
 #include "atom.h"
 #include "fragment.h"
-#include "cycle.h"
 #include "settings.h"
 #include "document.h"
 #include "theme.h"
@@ -36,8 +35,14 @@
 #include <canvas/gcp-canvas-line.h>
 #include <canvas/gcp-canvas-bpath.h>
 #include <canvas/gcp-canvas-polygon.h>
+#include <gcu/cycle.h>
+#include <gcu/objprops.h>
 #include <glib/gi18n-lib.h>
-#include <math.h>
+#include <cmath>
+#include <cstring>
+
+using namespace gcu;
+using namespace std;
 
 namespace gcp {
 
@@ -85,49 +90,6 @@ double Bond::GetAngle2D (Atom* pAtom)
 	return HUGE_VAL;
 }
 
-double Bond::GetAngle2DRad (Atom* pAtom)
-{
-	double x1, y1, x2, y2;
-	m_Begin->GetCoords (&x1, &y1);
-	m_End->GetCoords (&x2, &y2);
-	x2 -= x1;
-	y2 -= y1;
-	double length = square (x2) + square (y2);
-	if (length == 0.0)
-		return HUGE_VAL;
-	if (pAtom == m_Begin)
-		return atan2 (-y2, x2);
-	else if (pAtom == m_End)
-		return atan2 (y2, -x2);
-	return HUGE_VAL;
-}
-
-Cycle* Bond::GetFirstCycle (std::list<Cycle*>::iterator& i, Cycle * pCycle)
-{
-	i = m_Cycles.begin ();
-	return GetNextCycle (i, pCycle);
-}
-
-Cycle* Bond::GetNextCycle (std::list<Cycle*>::iterator& i, Cycle * pCycle)
-{
-	if (*i == pCycle)
-		i++;
-	if (i == m_Cycles.end ())
-		return NULL;
-	pCycle = *i;
-	i++;
-	return pCycle;
-}
-
-bool Bond::IsInCycle (Cycle* pCycle)
-{
-	std::list<Cycle*>::iterator i, end = m_Cycles.end ();
-	for (i = m_Cycles.begin (); i != end; i++)
-		if ((*i) == pCycle)
-			return true;
-	return false;
-}
-
 bool Bond::GetLine2DCoords (unsigned Num, double* x1, double* y1, double* x2, double* y2)
 {
 	if ((Num == 0) || (Num > m_order))
@@ -164,7 +126,7 @@ bool Bond::GetLine2DCoords (unsigned Num, double* x1, double* y1, double* x2, do
 			Cycle* pCycle;
 			if (IsCyclic() > 1) {
 				//Search prefered cycle
-				std::list<Cycle*>::iterator i = m_Cycles.begin (), end = m_Cycles.end ();
+				list<Cycle*>::iterator i = m_Cycles.begin (), end = m_Cycles.end ();
 				pCycle = *i;
 				for (; i != end; i++)
 					if (pCycle->IsBetterForBonds (*i))
@@ -288,6 +250,8 @@ bool Bond::LoadNode (xmlNodePtr node)
 
 void Bond::IncOrder (int n)
 {
+	if (!m_Begin || !m_End)
+		return;
 	if (!((Atom*) GetAtom(0))->AcceptNewBonds () ||
 		!((Atom*) GetAtom (1))->AcceptNewBonds ())
 		m_order = 1;
@@ -304,6 +268,8 @@ void Bond::IncOrder (int n)
 double Bond::GetDist (double x, double y)
 {
 	double x1, y1, x2, y2, l, d1, d2;
+	if (!m_Begin || !m_End)
+		return G_MAXDOUBLE;
 	Theme *Theme = dynamic_cast <Document *> (GetDocument ())->GetTheme ();
 	double BondDist = Theme->GetBondDist () / Theme->GetZoomFactor ();
 	m_Begin->GetCoords (&x1, &y1);
@@ -324,14 +290,14 @@ double Bond::GetDist (double x, double y)
 
 void Bond::AddCycle (Cycle* pCycle)
 {
-	m_Cycles.push_back (pCycle);
+	gcu::Bond::AddCycle (pCycle);
 	if ((m_order == 2) && m_CoordsCalc)
 		SetDirty();
 }
 
 void Bond::RemoveCycle (Cycle* pCycle)
 {
-	m_Cycles.remove (pCycle);
+	gcu::Bond::RemoveCycle (pCycle);
 	if ((m_order == 2) && m_CoordsCalc)
 		SetDirty();
 }
@@ -346,7 +312,7 @@ void Bond::SetDirty ()
 
 void Bond::RemoveAllCycles ()
 {
-	m_Cycles.clear ();
+	gcu::Bond::RemoveAllCycles ();
 	if (m_order == 2) {
 		Document *pDoc = (Document*) GetDocument ();
 		if (pDoc)
@@ -373,20 +339,14 @@ void Bond::Revert ()
 	m_End = pAtom;
 }
 
-double Bond::Get2DLength ()
-{
-	double x1, y1, x2, y2;
-	m_Begin->GetCoords (&x1, &y1);
-	m_End->GetCoords (&x2, &y2);
-	return sqrt (square (x1 - x2) + square (y1 - y2));
-}
-
 void Bond::SetSelected (GtkWidget* w, int state)
 {
 	if (!m_order)
 		return;
 	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (w), "data");
 	GnomeCanvasGroup* group = pData->Items[this];
+	if (!group)
+		return; // this should not occur, but it might crash if it does occur after loading a bad file.
 	gchar const *color;
 	switch (state) {	
 	case SelStateUnselected:
@@ -425,10 +385,12 @@ void Bond::Add (GtkWidget* w)
 	if (!w)
 		return;
 	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (w), "data");
+	if (pData->Items[this] != NULL)
+		return;
 	Theme *pTheme = pData->m_View->GetDoc ()->GetTheme ();
 	gcu::Atom *pAtom0, *pAtom1;
 	if (!(pAtom0 = GetAtom (0)))
-		return; 
+		return;
 	if (!(pAtom1 = GetAtom (1)))
 		return;
 	unsigned char order = GetOrder ();
@@ -436,7 +398,6 @@ void Bond::Add (GtkWidget* w)
 		return;
 	GnomeCanvasGroup* group;
 	GnomeCanvasItem* item = NULL;
-	pData = (WidgetData*) g_object_get_data (G_OBJECT (w), "data");
 	group = GNOME_CANVAS_GROUP (gnome_canvas_item_new (pData->Group, gnome_canvas_group_ext_get_type (), NULL));
 	g_signal_connect (G_OBJECT (group), "event", G_CALLBACK (on_event), w);
 	g_object_set_data (G_OBJECT (group), "object", this);
@@ -724,6 +685,8 @@ void Bond::Update(GtkWidget* w)
 	if (!w || !m_order)
 		return;
 	WidgetData* pData = (WidgetData*) g_object_get_data(G_OBJECT(w), "data");
+	if (pData->Items[this] == NULL)
+		return;
 	Theme *pTheme = pData->m_View->GetDoc ()->GetTheme ();
 	bool result = false;
 	if (m_Crossing.size () > 0) {
@@ -735,6 +698,10 @@ void Bond::Update(GtkWidget* w)
 	GnomeCanvasPathDef* path;
 	void * obj;
 	GnomeCanvasGroup *group = pData->Items[this];
+	if (!group) {
+		Add (w);
+		return;
+	}
 	obj = g_object_get_data(G_OBJECT(group), "back");
 	if (result) {
 		path = BuildCrossingPathDef (pData);
@@ -924,4 +891,44 @@ void Bond::BringToFront ()
 	pView->Update (this);
 }
 
+struct BondTypeStruct {
+	BondType type;
+	bool invert;
+};
+static map<string, struct BondTypeStruct> BondTypesValues;
+bool Bond::SetProperty (unsigned property, char const *value)
+{
+	switch (property) {
+	case GCU_PROP_BOND_TYPE: {
+		if (BondTypesValues.size () == 0) {
+			struct BondTypeStruct s;
+			s.invert = false;
+			s.type = NormalBondType;
+			BondTypesValues["normal"] = s;
+			s.type = ForeBondType;
+			BondTypesValues["bold"] = s;
+			s.type = UpBondType;
+			BondTypesValues["wedge"] = s;
+			s.type = DownBondType;
+			BondTypesValues["hash"] = s;
+			s.invert = true;
+			s.type = UpBondType;
+			BondTypesValues["wedge-invert"] = s;
+			s.type = DownBondType;
+			BondTypesValues["hash-invert"] = s;
+		}
+		map<string, struct BondTypeStruct>::iterator it;
+		if ((it = BondTypesValues.find (value)) != BondTypesValues.end ()) {
+			m_type = (*it).second.type;
+			if ((*it).second.invert)
+				Revert ();
+		}
+		break;
+	}
+	default:
+		gcu::Bond::SetProperty (property, value);
+	}
+	return  true;
+}
+
 }	//	namespace gcp
