@@ -4,7 +4,7 @@
  * GChemPaint library
  * document.h 
  *
- * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -25,20 +25,16 @@
 #ifndef GCHEMPAINT_DOCUMENT_H
 #define GCHEMPAINT_DOCUMENT_H
 
+#include "operation.h"
+#include <gcu/document.h>
+#include <gcu/macros.h>
+#include <gcu/printable.h>
 #include <list>
 #include <map>
 #include <set>
 #include <string>
 #include <libxml/tree.h>
 #include <gtk/gtk.h>
-#include <libgnomeprint/gnome-print.h>
-#include <gcu/document.h>
-#include <gcu/macros.h>
-#include "atom.h"
-#include "fragment.h"
-#include "bond.h"
-#include "molecule.h"
-#include "operation.h"
 
 namespace OpenBabel
 {
@@ -55,8 +51,13 @@ class View;
 class Application;
 class Window;
 class Theme;
+class Residue;
+class Atom;
+class Bond;
+class Fragment;
+class Molecule;
 
-class Document: public gcu::Document
+	class Document: public gcu::Document, public gcu::Printable
 {
 	//Constructor and destructor
 public:
@@ -80,7 +81,7 @@ public:
 	const gchar* GetLabel ();
 	void SetFileName (std::string const &, const gchar *mime_type);
 	const gchar* GetFileName () {return m_filename;}
-	void Print (GnomePrintContext *pc, gdouble width, gdouble height);
+	void DoPrint (GtkPrintOperation *print, GtkPrintContext *context);
 	void AddObject (Object* pObject);
 	void AddAtom (Atom* pAtom);
 	void AddFragment (Fragment* pFragment);
@@ -88,7 +89,6 @@ public:
 	void ParseXMLTree (xmlDocPtr xml);
 	void LoadObjects (xmlNodePtr node);
 	xmlDocPtr BuildXMLTree ();
-	void NotifyDirty (Bond* pBond) {m_DirtyObjects.insert (pBond);}
 	void Update ();
 	void Remove (Object*);
 	void Remove (const char* Id);
@@ -120,11 +120,15 @@ public:
 	bool GetReadOnly () {return m_bReadOnly;}
 	virtual double GetYAlign ();
 	Window *GetWindow () {return m_Window;}
+	GtkWindow *GetGtkWindow ();
 	void SetTheme (Theme *theme);
 	bool OnSignal (gcu::SignalId Signal, gcu::Object *Child);
 	void SetDirty (bool isDirty = true);
 	void OnThemeNamesChanged ();
 	double GetMedianBondLength ();
+	bool SetProperty (unsigned property, char const *value);
+	void SaveResidue (Residue const *r, xmlNodePtr node);
+	void SetLoading (bool loading) {m_bIsLoading = loading;}
 
 private:
 	void RemoveAtom (Atom* pAtom);
@@ -138,7 +142,6 @@ private:
 	gchar *m_title;
 	gchar *m_label;
 	gchar *m_comment, *m_author, *m_mail;
-	std::set<gcu::Object*> m_DirtyObjects;
 	bool m_bIsLoading, m_bUndoRedo, m_bReadOnly;
 	std::string m_FileType;
 	bool m_bWriteable;
@@ -149,6 +152,7 @@ private:
 	Window *m_Window;
 	unsigned long m_OpID; // last operation ID
 	unsigned m_LastStackSize; // undo list size when last saved
+	std::set<Residue const *> m_SavedResidues;
 
 /* Theme is not really a read only property, but we provide a special Set
 method */
