@@ -4,7 +4,7 @@
  * GChemPaint library
  * molecule.cc 
  *
- * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -29,12 +29,15 @@
 #include "application.h"
 #include "tool.h"
 #include "stringdlg.h"
+#include <gcu/chain.h>
 #include <glib/gi18n-lib.h>
 #include <unistd.h>
 #include <openbabel/obconversion.h>
 #include <clocale>
 #include <cmath>
 #include <cstring>
+#include <map>
+#include <set>
 
 using namespace gcu;
 using namespace OpenBabel;
@@ -78,18 +81,15 @@ static void do_open_in_calc (Molecule* pMol)
 	pMol->OpenCalc ();
 }
 
-Molecule::Molecule (TypeId Type): Object (Type)
+Molecule::Molecule (TypeId Type): gcu::Molecule (Type)
 {
 	m_Alignment = NULL;
 	m_Changed = true;
 	m_IsResidue = false;
 }
 
-Molecule::Molecule (Atom* pAtom): Object (MoleculeType)
+Molecule::Molecule (Atom* pAtom): gcu::Molecule  (pAtom)
 {
-	AddAtom (pAtom);
-	Chain* pChain = new Chain (this, pAtom); //will find the cycles
-	delete pChain;
 	m_Alignment = NULL;
 	m_Changed = true;
 	m_IsResidue = false;
@@ -97,24 +97,26 @@ Molecule::Molecule (Atom* pAtom): Object (MoleculeType)
 
 Molecule::~Molecule ()
 {
-	std::list<Bond*>::iterator n, end = m_Bonds.end ();
-	for (n = m_Bonds.begin (); n != end; n++)
-		(*n)->RemoveAllCycles ();
-	while (!m_Cycles.empty ()) {
-		delete m_Cycles.front ();
-		m_Cycles.pop_front ();
+}
+
+void Molecule::AddChild (Object* object)
+{
+	switch (object->GetType ()) {
+	case FragmentType: {
+		Fragment *fragment = reinterpret_cast<Fragment *> (object);
+		m_Fragments.remove (fragment);
+		AddFragment (fragment);
+		break;
 	}
-	while (!m_Chains.empty ()) {
-		delete m_Chains.front ();
-		m_Chains.pop_front ();
+	default:
+		gcu::Molecule::AddChild (object);
+		break;
 	}
 }
 
-void Molecule::AddAtom (Atom* pAtom)
+void Molecule::AddAtom (gcu::Atom* pAtom)
 {
-	m_Atoms.remove (pAtom); // avoid duplicates
-	m_Atoms.push_back (pAtom);
-	AddChild (pAtom);
+	gcu::Molecule::AddAtom (pAtom);
 	if (!pAtom->GetZ ())
 		m_IsResidue = true;
 }
@@ -122,15 +124,14 @@ void Molecule::AddAtom (Atom* pAtom)
 void Molecule::AddFragment (Fragment* pFragment)
 {
 	m_Fragments.push_back (pFragment);
-	AddChild (pFragment);
+	Object::AddChild (pFragment);
 } 
 
-void Molecule::AddBond (Bond* pBond)
+void Molecule::AddBond (gcu::Bond* pBond)
 {
 	if (pBond->GetAtom (0) && pBond->GetAtom (1))
-		CheckCrossings (pBond);
-	m_Bonds.push_back (pBond);
-	AddChild (pBond);
+		CheckCrossings (reinterpret_cast <Bond *> (pBond));
+	gcu::Molecule::AddBond (pBond);
 	EmitSignal (OnChangedSignal);
 }
 
@@ -139,25 +140,16 @@ void Molecule::Remove (Object* pObject)
 	if (pObject == m_Alignment)
 		m_Alignment = NULL;
 	switch (pObject->GetType ()) {
-	case AtomType:
-		m_Atoms.remove ((Atom*) pObject);
-		break;
-	case gcu::BondType:
-		m_Bonds.remove ((Bond*) pObject);
-		break;
 	case FragmentType:
 		m_Fragments.remove ((Fragment*) pObject);
 		break;
+	default:
+		gcu::Molecule::Remove (pObject);
+		break;
 	}
 	pObject->SetParent (GetParent ());
 }
 
-void Molecule::UpdateCycles (Bond* pBond)	//FIXME: function not totally implemented
-{
-	Chain* pChain = new Chain (this, pBond); //will find the cycles
-	delete pChain;
-}
-
 typedef struct {
 	Atom *pAtom;
 	int sb; // number of shared bonds
@@ -172,7 +164,7 @@ bool Molecule::Merge (Molecule* pMolecule, bool RemoveDuplicates)
 	Chain* pChain;
 	Cycle* pCycle;
 	if (RemoveDuplicates) {
-		std::list<Atom*>::iterator i = m_Atoms.begin (), end = m_Atoms.end ();
+		std::list<gcu::Atom*>::iterator i = m_Atoms.begin (), end = m_Atoms.end ();
 		double x, y, x0, y0, x1, y1;
 		MergedAtom *ma;
 		Bond *b0, *b1;
@@ -212,7 +204,7 @@ bool Molecule::Merge (Molecule* pMolecule, bool RemoveDuplicates)
 						}
 					}
 				}
-				AtomMap[*i] = ma;
+				AtomMap[reinterpret_cast <Atom *> (*i)] =  ma;
 			}
 		}
 		// Now check if merging is possible for each shared atom.
@@ -311,7 +303,7 @@ bool Molecule::Merge (Molecule* pMolecule, bool RemoveDuplicates)
 		if (!DoMerge) return false;
 	}
 	while (!pMolecule->m_Atoms.empty ()) {
-		pAtom = pMolecule->m_Atoms.front ();
+		pAtom =  reinterpret_cast <Atom *> (pMolecule->m_Atoms.front ());
 		AddAtom (pAtom);
 		pMolecule->m_Atoms.pop_front ();
 	}
@@ -321,7 +313,7 @@ bool Molecule::Merge (Molecule* pMolecule, bool RemoveDuplicates)
 		pMolecule->m_Fragments.pop_front ();
 	}
 	while (!pMolecule->m_Bonds.empty ()) {
-		pBond = pMolecule->m_Bonds.front ();
+		pBond =  reinterpret_cast <Bond *> (pMolecule->m_Bonds.front ());
 		AddBond (pBond);
 		pMolecule->m_Bonds.pop_front ();
 	}
@@ -416,11 +408,15 @@ bool Molecule::Load (xmlNodePtr node)
 		CheckCrossings ((Bond*) pObject);
 	}
 	if (!m_Atoms.empty ()) {
-		Atom* pAtom = m_Atoms.front ();
-		std::list<Atom*>::iterator i = m_Atoms.begin ();
+		Atom* pAtom =  reinterpret_cast <Atom *> (m_Atoms.front ());
+		list<gcu::Atom*>::iterator i = m_Atoms.begin ();
 		i++;
 		for (; i != m_Atoms.end (); i++)
 			(*i)->SetParent (NULL);
+		// erase cycles
+		list<gcu::Bond*>::iterator j, jend = m_Bonds.end ();
+		for (j = m_Bonds.begin (); j != jend; j++)
+			(*j)->RemoveAllCycles ();
 		Chain* pChain = new Chain (this, pAtom); //will find the cycles
 		delete pChain;
 	}
@@ -435,27 +431,6 @@ bool Molecule::Load (xmlNodePtr node)
 	return true;
 }
 
-void Molecule::UpdateCycles ()
-{
-	Lock (true);
-	std::list<Bond*>::iterator n, end = m_Bonds.end ();
-	for (n = m_Bonds.begin (); n != end; n++)
-		(*n)->RemoveAllCycles ();
-	while (!m_Cycles.empty()) {
-		delete m_Cycles.front ();
-		m_Cycles.pop_front ();
-	}
-	if (!m_Atoms.empty()) {
-		std::list<Atom*>::iterator i = m_Atoms.begin (), end = m_Atoms.end ();
-		i++;
-		for (; i != end; i++)
-			(*i)->SetParent(NULL);
-		Chain* pChain = new Chain (this, *(m_Atoms.begin ())); //will find the cycles
-		delete pChain;
-	}
-	Lock (false);
-}
-
 void Molecule::Clear ()
 {
 	m_Bonds.clear ();
@@ -476,16 +451,16 @@ void Molecule::SetSelected (GtkWidget* w, int state)
 void Molecule::Transform2D (Matrix2D& m, double x, double y)
 {
 	Object::Transform2D (m, x, y);
-	std::list<Atom*>::iterator i = m_Atoms.begin ();
+	std::list<gcu::Atom*>::iterator i = m_Atoms.begin ();
 	for (; i != m_Atoms.end (); i++)
-	if (((*i)->GetZ () != 6) && (*i)->GetAttachedHydrogens () &&
-		(*i)->GetBondsNumber ()) (*i)->Update ();
+	if (((*i)->GetZ () != 6) &&  reinterpret_cast <Atom *> (*i)->GetAttachedHydrogens () &&
+		(*i)->GetBondsNumber ())  reinterpret_cast <Atom *> (*i)->Update ();
 }
 
 Object* Molecule::GetAtomAt (double x, double y, double z)
 {
 	// Make use of Bond::GetAtomAt
-	std::list<Bond*>::iterator n, end = m_Bonds.end ();
+	std::list<gcu::Bond*>::iterator n, end = m_Bonds.end ();
 	Object* pObj = NULL;
 	for (n = m_Bonds.begin(); n != end; n++)
 		if ((pObj = (*n)->GetAtomAt (x, y)))
@@ -498,9 +473,9 @@ double Molecule::GetYAlign ()
 	if (m_Alignment)
 		return m_Alignment->GetYAlign ();
 	double y, maxy = - DBL_MAX, miny = DBL_MAX;
-	std::list<Atom*>::iterator i = m_Atoms.begin (), end = m_Atoms.end ();
+	std::list<gcu::Atom*>::iterator i = m_Atoms.begin (), end = m_Atoms.end ();
 	for (; i != end; i++) {
-		y = (*i)->GetYAlign ();
+		y =  reinterpret_cast <Atom *> (*i)->GetYAlign ();
 		if (y < miny)
 			miny = y;
 		if (y > maxy)
@@ -521,7 +496,6 @@ bool Molecule::BuildContextualMenu (GtkUIManager *UIManager, Object *object, dou
 {
 	if (m_IsResidue)
 		return false;
-	bool result = false;
 	GtkActionGroup *group = gtk_action_group_new ("molecule");
 	GtkAction *action;
 	action = gtk_action_new ("Molecule", _("Molecule"), NULL, NULL);
@@ -557,13 +531,12 @@ bool Molecule::BuildContextualMenu (GtkUIManager *UIManager, Object *object, dou
 		gtk_action_group_add_action (group, action);
 		g_object_unref (action);
 		gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menuitem action='smiles'/></menu></popup></ui>", -1, NULL);
-		action = gtk_action_new ("calc", _("Open in Calculator"), NULL, NULL);
-		g_signal_connect_swapped (action, "activate", G_CALLBACK (do_open_in_calc), this);
-		gtk_action_group_add_action (group, action);
-		g_object_unref (action);
-		gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menuitem action='calc'/></menu></popup></ui>", -1, NULL);
-		result = true;
 	}
+	action = gtk_action_new ("calc", _("Open in Calculator"), NULL, NULL);
+	g_signal_connect_swapped (action, "activate", G_CALLBACK (do_open_in_calc), this);
+	gtk_action_group_add_action (group, action);
+	g_object_unref (action);
+	gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menuitem action='calc'/></menu></popup></ui>", -1, NULL);
 	if (m_Bonds.size ()) {
 		action = gtk_action_new ("select-align", _("Select alignment item"), NULL, NULL);
 		g_signal_connect (action, "activate", G_CALLBACK (do_select_alignment), this);
@@ -571,12 +544,10 @@ bool Molecule::BuildContextualMenu (GtkUIManager *UIManager, Object *object, dou
 		gtk_action_group_add_action (group, action);
 		g_object_unref (action);
 		gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menuitem action='select-align'/></menu></popup></ui>", -1, NULL);
-		result = true;
 	}
-	if (result)
-		gtk_ui_manager_insert_action_group (UIManager, group, 0);
+	gtk_ui_manager_insert_action_group (UIManager, group, 0);
 	g_object_unref (group);
-	return result | Object::BuildContextualMenu (UIManager, object, x, y);
+	return Object::BuildContextualMenu (UIManager, object, x, y);
 }
 
 void Molecule::ExportToGhemical ()
@@ -626,8 +597,8 @@ void Molecule::BuildOBMol (OBMol &Mol)
 	double xav = 0., yav = 0., zf;
 	unsigned n = m_Atoms.size ();
 	map<string, unsigned> AtomTable;
-	list<Atom*>::iterator ia, enda = m_Atoms.end ();
-	list<Bond*> BondList;
+	list<gcu::Atom*>::iterator ia, enda = m_Atoms.end ();
+	list<gcu::Bond*> BondList;
 	double x, y, z;
 	for (ia = m_Atoms.begin (); ia != enda; ia++) {
 		(*ia)->GetCoords(&x, &y, &z);
@@ -644,7 +615,7 @@ void Molecule::BuildOBMol (OBMol &Mol)
 	Mol.BeginModify ();
 	Mol.ReserveAtoms (n);
 	for (ia = m_Atoms.begin (); ia != enda; ia++) {
-		pgAtom = *ia;
+		pgAtom =  reinterpret_cast <Atom *> (*ia);
 		AtomTable [pgAtom->GetId ()] = index;
 		obAtom.SetIdx (index++);
 		obAtom.SetAtomicNum (pgAtom->GetZ());
@@ -669,7 +640,7 @@ void Molecule::BuildOBMol (OBMol &Mol)
 		Mol.AddAtom (obAtom);
 		obAtom.Clear ();
 	}
-	list<Bond*>::iterator j, endb = m_Bonds.end ();
+	list<gcu::Bond*>::iterator j, endb = m_Bonds.end ();
 	int start, end, order;
 	for (j = m_Bonds.begin (); j != endb; j++)
 	{
@@ -686,8 +657,8 @@ void Molecule::BuildOBMol2D (OBMol &Mol)
 	double xav = 0., yav = 0.;
 	unsigned n = m_Atoms.size ();
 	map<string, unsigned> AtomTable;
-	list<Atom*>::iterator ia, enda = m_Atoms.end ();
-	list<Bond*> BondList;
+	list<gcu::Atom*>::iterator ia, enda = m_Atoms.end ();
+	list<gcu::Bond*> BondList;
 	double x, y, z;
 	for (ia = m_Atoms.begin (); ia != enda; ia++) {
 		(*ia)->GetCoords (&x, &y, &z);
@@ -704,7 +675,7 @@ void Molecule::BuildOBMol2D (OBMol &Mol)
 	Mol.ReserveAtoms (n);
 	Mol.SetDimension (2);
 	for (ia = m_Atoms.begin (); ia != enda; ia++) {
-		pgAtom = *ia;
+		pgAtom =  reinterpret_cast <Atom *> (*ia);
 		AtomTable [pgAtom->GetId ()] = index;
 		obAtom.SetIdx (index++);
 		obAtom.SetAtomicNum (pgAtom->GetZ());
@@ -714,7 +685,7 @@ void Molecule::BuildOBMol2D (OBMol &Mol)
 		Mol.AddAtom (obAtom);
 		obAtom.Clear ();
 	}
-	list<Bond*>::iterator j, endb = m_Bonds.end ();
+	list<gcu::Bond*>::iterator j, endb = m_Bonds.end ();
 	int start, end, order, flag;
 	for (j = m_Bonds.begin (); j != endb; j++)
 	{
@@ -832,14 +803,14 @@ bool Molecule::OnSignal (SignalId Signal, Object *Child)
 
 void Molecule::OpenCalc ()
 {
-	list<Atom*>::iterator ia, enda = m_Atoms.end ();
+	list<gcu::Atom*>::iterator ia, enda = m_Atoms.end ();
 	ostringstream ofs;
 	int nH;
 	// FIXME: the following line should be edited for stable releases
 	ofs << "gchemcalc-unstable ";
 	for (ia = m_Atoms.begin(); ia != enda; ia++) {
 		ofs << (*ia)->GetSymbol();
-		nH = (*ia)->GetAttachedHydrogens ();
+		nH = reinterpret_cast <Atom *> (*ia)->GetAttachedHydrogens ();
 		if (nH > 0) {
 			ofs << "H" << nH;
 		}
@@ -850,22 +821,14 @@ void Molecule::OpenCalc ()
 void Molecule::CheckCrossings (Bond *pBond)
 {
 	View *pView = reinterpret_cast<Document*> (GetDocument ())->GetView ();
-	list<Bond*>::iterator i, iend = m_Bonds.end ();
+	list<gcu::Bond*>::iterator i, iend = m_Bonds.end ();
 	for (i = m_Bonds.begin (); i != iend; i++)
-		if (((*i) != pBond) && (*i)->IsCrossing (pBond)) {
+		if (((*i) != pBond) && reinterpret_cast <Bond *> (*i)->IsCrossing (pBond)) {
 			pView->Update (pBond);
 			pView->Update (*i);
 		}
 }
 
-void Molecule::Add (GtkWidget* w)
-{
-	/* Now that Object::Add adds children, we need to override this method
-	because a molecule children (atoms, bonds, and fragments) are added to
-	the widget by the document. FIXME! FIXME! FIXME! CHANGE THIS OLD WEIRD CODE!
-	*/
-}
-
 char const *Molecule::GetInChI ()
 {
 	if (m_Changed)
@@ -878,14 +841,14 @@ std::string Molecule::GetRawFormula ()
 	ostringstream ofs;
 
 	if (!m_Fragments.size ()) {
-		// we do not support fragmentsat the moment
+		// we do not support fragments at the moment
 		map<string, int> elts;
-		list<Atom*>::iterator ia, enda = m_Atoms.end ();
+		list<gcu::Atom*>::iterator ia, enda = m_Atoms.end ();
 		for (ia = m_Atoms.begin(); ia != enda; ia++) {
 			if ((*ia)->GetZ () == 0)
 				continue;
 			elts[(*ia)->GetSymbol ()]++;
-			elts["H"] += (*ia)->GetAttachedHydrogens ();
+			elts["H"] += reinterpret_cast <Atom *> (*ia)->GetAttachedHydrogens ();
 		}
 		if (elts["C"] > 0) {
 			ofs << "C" << elts["C"];
@@ -903,4 +866,25 @@ std::string Molecule::GetRawFormula ()
 	return ofs.str ();
 }
 
+void Molecule::OnLoaded ()
+{
+	UpdateCycles ();
+}
+
+bool Molecule::operator== (Molecule const& molecule)
+{
+	// first examine each atom of each molecule and sort by Z.
+	map<int, set<gcu::Atom*> > atoms1, atoms2;
+	list<gcu::Atom*>::const_iterator ia, enda = m_Atoms.end ();
+	for (ia = m_Atoms.begin ();  ia != enda; ia++)
+		atoms1[(*ia)->GetZ ()].insert (*ia);
+	enda = molecule.m_Atoms.end ();
+	for (ia = molecule.m_Atoms.begin ();  ia != enda; ia++)
+		atoms2[(*ia)->GetZ ()].insert (*ia);
+	if (atoms1.size () != atoms2.size ())
+		return false;
+	// TODO: write this code
+	return true;
+}
+
 }	//	namespace gcp
