@@ -33,10 +33,14 @@
 #include "theme.h"
 #include "view.h"
 #include "widgetdata.h"
+#include <gcu/objprops.h>
 #include <canvas/gcp-canvas-group.h>
 #include <canvas/gcp-canvas-line.h>
 #include <glib/gi18n-lib.h>
 #include <cmath>
+#include <cstring>
+
+using namespace gcu;
 
 namespace gcp {
 
@@ -47,6 +51,7 @@ ReactionArrow::ReactionArrow (Reaction* react, unsigned Type): Arrow (ReactionAr
 	m_Start = m_End = NULL;
 	if (react)
 		react->AddChild (this);
+	m_TypeChanged = false;
 }
 
 
@@ -151,6 +156,8 @@ bool ReactionArrow::Load (xmlNodePtr node)
 void ReactionArrow::Add (GtkWidget* w)
 {
 	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (w), "data");
+	if (pData->Items[this] != NULL)
+		return;
 	Theme *pTheme = pData->m_View->GetDoc ()->GetTheme ();
 	GnomeCanvasPoints *points = gnome_canvas_points_new (2);
 	GnomeCanvasGroup* group = GNOME_CANVAS_GROUP(gnome_canvas_item_new (pData->Group, gnome_canvas_group_ext_get_type (), NULL));
@@ -282,6 +289,7 @@ void ReactionArrow::Update (GtkWidget* w)
 	}
 	if (m_TypeChanged) {
 		gtk_object_destroy (GTK_OBJECT (group));
+		pData->Items[this] = NULL;
 		Add (w);
 		m_TypeChanged = false;
 		return;
@@ -302,6 +310,7 @@ void ReactionArrow::Update (GtkWidget* w)
 								NULL);
 			break;
 		case ReversibleArrow:
+		case FullReversibleArrow:
 			double dAngle = atan (- m_height / m_width);
 			if (m_width < 0) dAngle += M_PI;
 			points->coords[0] = m_x * pTheme->GetZoomFactor () - pTheme->GetArrowDist () / 2 * sin (dAngle);
@@ -482,4 +491,16 @@ bool ReactionArrow::OnSignal (SignalId Signal, Object *Child)
 	return true;
 }
 
+bool ReactionArrow::SetProperty (unsigned property, char const *value)
+{
+	switch (property) {
+	case GCU_PROP_REACTION_ARROW_TYPE:
+		m_Type = (strcmp (value, "double"))? ReversibleArrow: SimpleArrow;
+		break;
+	default:
+		return Arrow::SetProperty (property, value);
+	}
+	return true;
+}
+
 }	//	namespace gcp
