@@ -4,7 +4,7 @@
  * GChemPaint library
  * residue.h 
  *
- * Copyright (C) 2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2007-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -31,6 +31,9 @@
 namespace gcp
 {
 
+class Document;
+class Molecule;
+
 class Residue: public gcu::Residue
 {
 public:
@@ -41,9 +44,12 @@ public:
 	virtual void Load (xmlNodePtr node, bool ro);
 
 private:
+	Document *m_Doc;
+
 GCU_RO_PROP (bool, ReadOnly);
 GCU_RO_PROP (xmlNodePtr, Node);
 GCU_RO_PROP (xmlNodePtr, MolNode);
+GCU_RO_POINTER_PROP (Molecule, Molecule);
 };
 
 }	//	namespace gcp
