@@ -25,6 +25,7 @@
 #ifndef GCHEMPAINT_TEXT_H
 #define GCHEMPAINT_TEXT_H
 
+#include <gcu/macros.h>
 #include <gcu/object.h>
 #include <libxml/tree.h>
 #include <gtk/gtk.h>
@@ -50,11 +51,14 @@ public:
 	void Update (GtkWidget* w);
 	void SetSelected (GtkWidget* w, int state);
 	bool OnChanged (bool save);
-	void Transform2D (Matrix2D& m, double x, double y);
+	void Transform2D (gcu::Matrix2D& m, double x, double y);
 	bool OnEvent (GdkEvent *event);
 	void GetSize (double& x, double& y) {x = m_length; y = m_height;}
 	double GetYAlign ();
 	void SetText (char const *text) {m_buf = text;}
+	bool SetProperty (unsigned property, char const *value);
+	
+GCU_RO_PROP (PangoAlignment, Align);
 };
 
 }	//	namespace gcp
