@@ -29,6 +29,9 @@
 #include <map>
 #include <set>
 #include <string>
+#ifdef HAVE_GO_CONF_SYNC
+#include <goffice/app/go-conf.h>
+#endif
 #include <gtk/gtkmain.h>
 #include <gtk/gtkwindow.h>
 #include <gtk/gtkrecentmanager.h>
@@ -39,6 +42,8 @@ namespace gcu {
 class Document;
 class Dialog;
 
+#define GCU_CONF_DIR "gchemutils"
+
 /*!\class Application gcu/application.h
 This class is a base class for applications. It provides some basic services.
 */
@@ -182,6 +187,12 @@ A default extension is appended to filename if none is detected.
 is not supported by GdkPixbuf.
 */
 	char const *GetPixbufTypeName (std::string& filename, char const *mime_type);
+	bool Load (std::string const &uri, const gchar *mime_type, Document* Doc);
+	bool Save (std::string const &uri, const gchar *mime_type, Document* Doc);
+	virtual Document *CreateNewDocument () {return NULL;}
+#ifdef HAVE_GO_CONF_SYNC
+	static GOConfNode *GetConfDir ();
+#endif
 
 protected:
 
@@ -204,6 +215,9 @@ private:
 	std::string CurDir;
 	std::string WebBrowser;
 	std::string MailAgent;
+#ifdef HAVE_GO_CONF_SYNC
+	static GOConfNode *m_ConfDir;
+#endif
 
 protected:
 /*!
