@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * gcu/chem3ddoc.cc
  *
- * Copyright (C) 2006-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2006-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -32,14 +32,20 @@
 #include <gcu/element.h>
 #include <openbabel/obconversion.h>
 #include <libgnomevfs/gnome-vfs.h>
+#include <gio/gio.h>
 #include <GL/gl.h>
 #include <libintl.h>
 #include <clocale>
 #include <fstream>
 #include <ostream>
 #include <sstream>
+#include <cstring>
 
-using namespace gcu;
+using namespace OpenBabel;
+using namespace std;
+
+namespace gcu
+{
 
 Chem3dDoc::Chem3dDoc (): GLDocument (NULL)
 {
@@ -59,30 +65,46 @@ Chem3dDoc::~Chem3dDoc ()
 
 void Chem3dDoc::Load (char const *uri, char const *mime_type)
 {
-	GnomeVFSHandle *handle;
-	GnomeVFSFileInfo *info = gnome_vfs_file_info_new ();
-	GnomeVFSResult result = gnome_vfs_open (&handle, uri, GNOME_VFS_OPEN_READ);
-	if (result != GNOME_VFS_OK) {
-		gnome_vfs_file_info_unref (info);
+	GVfs *vfs = g_vfs_get_default ();
+	GFile *file = g_vfs_get_file_for_uri (vfs, uri);
+	GError *error = NULL;
+	GFileInfo *info = g_file_query_info (file,
+										 ((mime_type)? G_FILE_ATTRIBUTE_STANDARD_SIZE:
+										 G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE","G_FILE_ATTRIBUTE_STANDARD_SIZE),
+										 G_FILE_QUERY_INFO_NONE,
+										 NULL, &error);
+	if (error) {
+		g_message ("GIO querry failed: %s", error->message);
+		g_error_free (error);
+		g_object_unref (file);
+		error = NULL;
 		return;
 	}
-	if (mime_type)
-		gnome_vfs_get_file_info_from_handle (handle, info, (GnomeVFSFileInfoOptions) 0);
-	else {
-		gnome_vfs_get_file_info_from_handle (handle, info,
-			(GnomeVFSFileInfoOptions)(GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
-								GNOME_VFS_FILE_INFO_FORCE_SLOW_MIME_TYPE));
-		if (!strncmp (info->mime_type, "text", 4))
-			gnome_vfs_get_file_info_from_handle (handle, info,
-				(GnomeVFSFileInfoOptions)GNOME_VFS_FILE_INFO_GET_MIME_TYPE);
+	if (!mime_type)
+		mime_type = g_file_info_get_content_type(info);
+	gsize size = g_file_info_get_size (info);
+	g_object_unref (info);
+	GInputStream *input = G_INPUT_STREAM (g_file_read (file, NULL, &error));
+	if (error) {
+		g_message ("GIO could not create the stream: %s", error->message);
+		g_error_free (error);
+		g_object_unref (file);
+		error = NULL;
+		return;
 	}
-	gchar *buf = new gchar[info->size + 1];
-	GnomeVFSFileSize n;
-	gnome_vfs_read (handle, buf, info->size, &n);
-	buf[info->size] = 0;
-	if (n == info->size) {
-		if (!mime_type)
-			mime_type = info->mime_type;
+	gchar *buf = new gchar[size + 1];
+	gsize n = g_input_stream_read (input, buf, size, NULL, &error);
+	if (error) {
+		g_message ("GIO could not read the file: %s", error->message);
+		g_error_free (error);
+		delete [] buf;
+		g_object_unref (input);
+		g_object_unref (file);
+		error = NULL;
+		return;
+	}
+	buf[size] = 0;
+	if (n == size) {
 		LoadData (buf, mime_type);
 		if (m_App) {
 			char *dirname = g_path_get_dirname (uri);
@@ -92,10 +114,9 @@ void Chem3dDoc::Load (char const *uri, char const *mime_type)
 	}
 	if (!strlen (m_Mol.GetTitle()))
 		m_Mol.SetTitle (g_path_get_basename (uri));
-	gnome_vfs_file_info_unref (info);
 	delete [] buf;
-	g_free (handle);
-	
+	g_object_unref (input);
+	g_object_unref (file);
 }
 
 void Chem3dDoc::LoadData (char const *data, char const *mime_type)
@@ -129,7 +150,7 @@ void Chem3dDoc::LoadData (char const *data, char const *mime_type)
 	vector3 v(-x0, -y0, -z0);
 	atom = m_Mol.BeginAtom (i);
 	while (atom) {
-		atom->GetVector () += v;
+		atom->SetVector (atom->GetVector () + v);
 		atom = m_Mol.NextAtom (i);
 	}
 	m_View->Update ();
@@ -305,15 +326,24 @@ void Chem3dDoc::Draw (Matrix &m)
 	dist = 0.;
 	map<OBAtom*, vector3> atomPos;
 	const gdouble* color;
-	vector3 v;
+	vector3 v, normal (0., 0., 1.);
 	Sphere sp (10);
-	glEnable (GL_RESCALE_NORMAL);
+	if (m_Display3D == WIREFRAME) {
+		float light_ambient[] = {1.0, 1.0, 1.0, 1.0};
+		glLightfv (GL_LIGHT0, GL_AMBIENT, light_ambient);
+	} else {
+		glEnable (GL_RESCALE_NORMAL);
+		float light_ambient[] = {.0, .0, .0, 1.0};
+		glLightfv (GL_LIGHT0, GL_AMBIENT, light_ambient);
+	}
 	while (atom) {
 		atomPos[atom] = v = m * atom->GetVector ();
 		Z = atom->GetAtomicNum ();
 		if (Z > 0) {
 			if (m_Display3D == CYLINDERS) {
 				R = .12;
+			} else if (m_Display3D == WIREFRAME) {
+				R = 0.;
 			} else {
 				R = etab.GetVdwRad (Z);
 				if (m_Display3D == BALL_AND_STICK)
@@ -325,13 +355,15 @@ void Chem3dDoc::Draw (Matrix &m)
 			color = gcu_element_get_default_color (Z);
 			if ((w = sqrt (x * x + y * y + z * z)) > dist - R)
 				dist = w + R;
-			glColor3d (color[0], color[1], color[2]) ;
-			sp.draw (v, R);
+			if (m_Display3D != WIREFRAME) {
+				glColor3d (color[0], color[1], color[2]);
+				sp.draw (v, R);
+			}
 		}
 		atom = m_Mol.NextAtom (i);
 	}
 	m_MaxDist = dist * 1.05;
-	if (m_Display3D == BALL_AND_STICK || m_Display3D == CYLINDERS) {
+	if (m_Display3D != SPACEFILL) {
 		Cylinder cyl (10);
 		m_MaxDist = dist * 1.05;
 		std::vector < OBEdgeBase * >::iterator j;
@@ -339,7 +371,10 @@ void Chem3dDoc::Draw (Matrix &m)
 		vector3 v0, v1;
 		double R1;
 		unsigned int Z1;
-		glEnable (GL_NORMALIZE);
+		if (m_Display3D == WIREFRAME)
+			sp.draw (v, 0.); // weird, this initializes something needed to see colors but what?
+		else
+			glEnable (GL_NORMALIZE);
 		while (bond) {
 			atom = bond->GetBeginAtom ();
 			v = m * atom->GetVector ();
@@ -359,12 +394,30 @@ void Chem3dDoc::Draw (Matrix &m)
 			v1 = m * atom->GetVector ();
 			v0 = v + (v1 - v) * (R / (R + R1));
 			color = gcu_element_get_default_color (Z);
-			glColor3d (color[0], color[1], color[2]) ;
-			cyl.draw (v, v0, .12);
+			glColor3d (color[0], color[1], color[2]);
+			if (m_Display3D == WIREFRAME) {
+				glBegin (GL_LINES);
+				glVertex3d (v.x (), v.y (), v.z());
+				glVertex3d (v0.x (), v0.y (), v0.z());
+				glEnd ();
+			} else if (m_Display3D == BALL_AND_STICK && bond->GetBondOrder () > 1)
+				cyl.drawMulti (v, v0, ((bond->GetBondOrder () > 2)? .07: 0.10), bond->GetBondOrder (), 0.15, normal);
+			else
+				cyl.draw (v, v0, .12);
 			color = gcu_element_get_default_color (Z1);
-			glColor3d (color[0], color[1], color[2]) ;
-			cyl.draw (v0, v1, .12);
+			glColor3d (color[0], color[1], color[2]);
+			if (m_Display3D == WIREFRAME) {
+				glBegin (GL_LINES);
+				glVertex3d (v0.x (), v0.y (), v0.z());
+				glVertex3d (v1.x (), v1.y (), v1.z());
+				glEnd ();
+			} else if (m_Display3D == BALL_AND_STICK && bond->GetBondOrder () > 1)
+				cyl.drawMulti (v0, v1, ((bond->GetBondOrder () > 2)? .07: 0.10), bond->GetBondOrder (), 0.15, normal);
+			else
+				cyl.draw (v0, v1, .12);
 			bond = m_Mol.NextBond (j);
 		}
 	}
 }
+
+}	//	namespace gcu
