@@ -28,12 +28,14 @@
 #include <GL/gl.h>
 #include <GL/glu.h>
 #include <cmath>
+#include <cstring>
 
 #define __max(x,y)  ((x) > (y)) ? (x) : (y)
 #define __min(x,y)  ((x) < (y)) ? (x) : (y)
 #define square(x)	((x)*(x))
 
-namespace gcu {
+namespace gcu
+{
 	
 char const *LineTypeName[] = {
 	"edges",
@@ -43,10 +45,6 @@ char const *LineTypeName[] = {
 	"unique"
 };
 
-}
-
-using namespace gcu;
-
 CrystalLine::CrystalLine()
 {
 	m_dx = m_dy = m_dz = m_dx2 = m_dy2 = m_dz2 = m_dl = m_dr = 0.0;
@@ -322,3 +320,5 @@ bool CrystalLine::Load (xmlNodePtr node)
 		return false;
 	return true;
 }
+
+}	//	namespace gcu
