@@ -25,6 +25,7 @@
 #include "config.h"
 #include "formula.h"
 #include "element.h"
+#include "residue.h"
 #include <glib/gi18n.h>
 #include <cctype>
 #include <cmath>
@@ -32,7 +33,10 @@
 #include <cstring>
 #include <sstream>
 
-using namespace gcu;
+using namespace std;
+
+namespace gcu
+{
 
 parse_error::parse_error(const string& __arg, int start, int length)
   : exception(), m_msg(__arg)
@@ -54,8 +58,6 @@ parse_error::what(int &start, int &length) const throw()
 	length = m_length;
 	return m_msg.c_str();
 }
-
-namespace gcu {
 	
 class FormulaElt
 {
@@ -64,6 +66,7 @@ public:
 	virtual ~FormulaElt ();
 	virtual string Markup ();
 	virtual void BuildRawFormula (map<int, int> &raw) = 0;
+	virtual int GetValence () = 0;
 	int stoich;
 };
 
@@ -72,8 +75,9 @@ class FormulaAtom: public FormulaElt
 public:
 	FormulaAtom (int Z);
 	virtual ~FormulaAtom ();
-	virtual string Markup ();
-	virtual void BuildRawFormula (map<int, int> &raw);
+	string Markup ();
+	void BuildRawFormula (map<int, int> &raw);
+	int GetValence ();
 	int elt;
 };
 
@@ -82,13 +86,25 @@ class FormulaBlock: public FormulaElt
 public:
 	FormulaBlock ();
 	virtual ~FormulaBlock ();
-	virtual string Markup ();
-	virtual void BuildRawFormula (map<int, int> &raw);
+	string Markup ();
+	void BuildRawFormula (map<int, int> &raw);
 	list<FormulaElt *> children;
+	int GetValence ();
 	int parenthesis;
 };
 
-}
+class FormulaResidue: public FormulaElt
+{
+public:
+	FormulaResidue (Residue const *res, char const *symbol, int Z);
+	virtual ~FormulaResidue ();
+	string Markup ();
+	void BuildRawFormula (map<int, int> &raw);
+	int GetValence ();
+	Residue const *residue;
+	string Symbol;
+	GCU_RO_PROP (int, Z);
+};
 
 FormulaElt::FormulaElt ()
 {
@@ -123,6 +139,11 @@ string FormulaAtom::Markup ()
 	return s;
 }
 
+int FormulaAtom::GetValence ()
+{
+	return Element::GetElement (elt)->GetDefaultValence ();
+}
+
 void FormulaAtom::BuildRawFormula (map<int, int> &raw)
 {
 	raw[elt] += stoich;
@@ -183,13 +204,73 @@ void FormulaBlock::BuildRawFormula (map<int, int> &raw)
 		raw[(*j).first] += stoich * (*j).second;}
 }
 
-static bool AnalString (char *sz, list<FormulaElt *> &result)
+int FormulaBlock::GetValence ()
+{
+	return -1; // FIXME !!!
+}
+
+FormulaResidue::FormulaResidue (Residue const *res, char const *symbol, int Z): FormulaElt()
+{
+	residue = res;
+	Symbol = symbol;
+	m_Z = Z;
+}
+	
+FormulaResidue::~FormulaResidue ()
+{
+}
+
+string FormulaResidue::Markup ()
+{
+	size_t n = Symbol.find ('-');
+	if (n != string::npos) {
+		string s = string ("<i>") + string (Symbol, 0, n) + "</i>" + string (Symbol, n);
+		return s;
+	} else
+		return Symbol;
+}
+
+void FormulaResidue::BuildRawFormula (map<int, int> &raw)
+{
+	std::map<int,int> const &m = residue->GetRawFormula ();
+	map<int, int>::const_iterator j, jend = m.end();
+	for (j = m.begin (); j != jend; j++){
+		raw[(*j).first] += stoich * (*j).second;}
+}
+
+int FormulaResidue::GetValence ()
+{
+	return 1; // residues with other valences are not currently supported
+}
+
+static bool AnalString (char *sz, list<FormulaElt *> &result, bool &ambiguous)
 {
 	if (*sz == 0)
 		return true;
 	int i = 0;
-	char sy[4];
+	char sy[Residue::MaxSymbolLength + 1];
+	Residue const *r = NULL;
+	bool amb = ambiguous, local_amb;
 	if (*sz) {
+		// search for any abbreviation starting sz
+		strncpy (sy, sz, Residue::MaxSymbolLength);
+		i = Residue::MaxSymbolLength;
+		while (i > 0) {
+			sy[i] = 0;
+			r = Residue::GetResidue (sy, &local_amb);
+			if (r)
+				break;
+			i--;
+		}
+		if (r) {
+			result.push_back (new FormulaResidue (r, sy, (local_amb? Element::Z (sy): 0)));
+			ambiguous |= local_amb;
+			if (AnalString (sz + i, result, ambiguous))
+				return true;
+			ambiguous = amb; // restore ambiguity state
+			delete result.back ();
+			result.pop_back ();
+		}
 		if (islower (*sz)) {
 			/* we might have some abbreviation around there */
 		}
@@ -208,7 +289,7 @@ static bool AnalString (char *sz, list<FormulaElt *> &result)
 			i = Element::Z (sy);
 			if (i > 0) {
 				result.push_back (new FormulaAtom (i));
-				if (AnalString (sz + 1, result))
+				if (AnalString (sz + 1, result, ambiguous))
 					return true;
 				delete result.back ();
 				result.pop_back ();
@@ -218,7 +299,7 @@ static bool AnalString (char *sz, list<FormulaElt *> &result)
 			i = Element::Z (sy);
 			if (i > 0) {
 				result.push_back (new FormulaAtom (i));
-				if (AnalString (sz + 2, result))
+				if (AnalString (sz + 2, result, ambiguous))
 					return true;
 				delete result.back ();
 				result.pop_back ();
@@ -230,7 +311,7 @@ static bool AnalString (char *sz, list<FormulaElt *> &result)
 			i = Element::Z (sy);
 			if (i > 0) {
 				result.push_back (new FormulaAtom (i));
-				if (AnalString (sz + 3, result))
+				if (AnalString (sz + 3, result, ambiguous))
 					return true;
 			}
 			return false;
@@ -244,7 +325,7 @@ static bool AnalString (char *sz, list<FormulaElt *> &result)
 				i = Element::Z (sy);
 				if (i > 0) {
 					result.push_back (new FormulaAtom (i));
-					if (AnalString (sz + 3, result))
+					if (AnalString (sz + 3, result, ambiguous))
 						return true;
 					delete result.back ();
 					result.pop_back ();
@@ -254,7 +335,7 @@ static bool AnalString (char *sz, list<FormulaElt *> &result)
 			i = Element::Z (sy);
 			if (i > 0) {
 				result.push_back (new FormulaAtom (i));
-				if (AnalString (sz + 2, result))
+				if (AnalString (sz + 2, result, ambiguous))
 					return true;
 				delete result.back ();
 				result.pop_back ();
@@ -263,7 +344,7 @@ static bool AnalString (char *sz, list<FormulaElt *> &result)
 			i = Element::Z (sy);
 			if (i > 0) {
 				result.push_back (new FormulaAtom (i));
-				if (AnalString (sz + 1, result))
+				if (AnalString (sz + 1, result, ambiguous))
 					return true;
 			}
 		}
@@ -275,7 +356,7 @@ Formula::Formula (string entry) throw (parse_error)
 {
 	Entry = entry;
 	Parse (Entry, Details);
-	m_WeightCached = false;
+	m_ConnectivityCached = m_WeightCached = false;
 }
 
 Formula::~Formula ()
@@ -355,13 +436,14 @@ void Formula::Clear ()
 	Markup = "";
 	Raw.clear ();
 	RawMarkup = "";
-	m_WeightCached = false;
+	m_ConnectivityCached = m_WeightCached = false;
 }
 
 void Formula::Parse (string &formula, list<FormulaElt *> &result) throw (parse_error)
 {
 	int i = 0, npo, size = formula.size (), j, k = 0; // parsing index, number of open parenthesis, string size
 	char c = 0, *sz, *end;
+	bool ambiguous = false;
 	while (i < size) {
 		if (formula[i] == '(' || formula[i] == '[' || formula[i] == '{') {
 			switch (formula[i]) {
@@ -404,15 +486,15 @@ void Formula::Parse (string &formula, list<FormulaElt *> &result) throw (parse_e
 			i = end - formula.c_str ();
 			if (i == j)
 				block->stoich = 1;
-		} else if (isalpha (formula[i])) {
+		} else if (isalpha (formula[i]) || formula[i] == '-') {
 			j = i + 1;
-			while (isalpha (formula[j]))
+			while (isalpha (formula[j]) || formula[j] == '-')
 				j++;
 			k = j - i;
 			sz = new char[k + 1];
-			sz = strndup (formula.c_str () + i, k);
+			strncpy (sz, formula.c_str () + i, k);
 			sz[k] = 0;
-			if (!AnalString (sz, result)) {
+			if (!AnalString (sz, result, ambiguous)) {
 				delete [] sz;
 				throw parse_error (_("Could not interpret the symbol list"), i, k);
 			}
@@ -430,32 +512,51 @@ void Formula::Parse (string &formula, list<FormulaElt *> &result) throw (parse_e
 		} else
 			throw parse_error (_("Invalid character"), i, 1);
 	}
+	if (ambiguous) {
+		int replaced = 1, max = 0;
+		// first count ambiguous symbols
+		list<FormulaElt *>::iterator it, end = result.end ();
+		for (it = result.begin (); it != end; it++) {
+			if (dynamic_cast <FormulaResidue *> (*it) != NULL)
+				max++;
+		}
+		if (!BuildConnectivity ()) {
+			// for now just replace all ambiguous residues
+			it = result.begin ();
+			FormulaResidue *res;
+			while (it != result.end ()) {
+				res = dynamic_cast <FormulaResidue *> (*it);
+				if (res && res->GetZ ()) {
+					printf("found ambiguous with Z=%d\n",res->GetZ());
+					FormulaAtom *elt = new FormulaAtom (res->GetZ());
+					elt->stoich =  res->stoich;
+					it = result.erase (it);
+					delete res;
+					it = result.insert (it, elt);
+				} else
+					it++;
+			}
+		}
+	}
 }
 
-double Formula::GetMolecularWeight (int &prec, bool &artificial)
+DimensionalValue Formula::GetMolecularWeight (bool &artificial)
 {
 	if (Raw.size () == 0) {
-		prec = 0;
-		return 0.;
+		return m_Weight;
 	}
 	if (!m_WeightCached) {
-		double atom_weight, delta = 0.;
-		int atom_prec;
-		m_Weight = 0;
-		m_WeightPrec = 16; // something much greater that possible for any element.
+		DimensionalValue atom_weight;
 		m_Artificial = false; // most formula don't have artificial elements
-		map<int,int>::iterator i, end = Raw.end ();
-		for (i = Raw.begin (); i != end; i++) {
-			atom_weight = Element::GetElement ((*i).first)->GetWeight (atom_prec);
-			if (atom_prec == 0)
+		map<int,int>::iterator i, end = Raw.end (), begin = Raw.begin ();
+		for (i = begin; i != end; i++) {
+			atom_weight = *Element::GetElement ((*i).first)->GetWeight ();
+			if (atom_weight.GetValue ().prec == 0)
 				m_Artificial = true;
-			delta += pow10 (-atom_prec) * (*i).second;
-			m_Weight += atom_weight * (*i).second;
+			m_Weight = (i == begin)? atom_weight * (*i).second: m_Weight + atom_weight * (*i).second;
 		}
-		m_WeightPrec = (int) ceil (-log10 (delta) - 1e-5);
 	}
 	m_WeightCached = true;
-	prec = m_WeightPrec;
 	artificial = m_Artificial;
 	return m_Weight;
 }
@@ -485,3 +586,11 @@ void Formula::CalculateIsotopicPattern (IsotopicPattern &pattern)
 		pat->Unref ();
 	}
 }
+
+bool Formula::BuildConnectivity ()
+{
+	// FIXME: write this function
+	return false;
+}
+
+}	//	namespace gcu
