@@ -27,8 +27,13 @@
 
 #include "macros.h"
 #include "matrix.h"
+#include "printable.h"
+#ifdef HAVE_GO_CONF_SYNC
+#include <goffice/app/go-conf.h>
+#else
+#include <gconf/gconf-client.h>
+#endif
 #include <gtk/gtkwidget.h>
-#include <libgnomeprint/gnome-print.h>
 #include <map>
 #include <string>
 
@@ -43,7 +48,7 @@ class GLDocument;
 View class based on OpenGL for rendering. Used to display 3d chemical structures
 such as molecules or crystals cells.
 */
-class GLView
+class GLView: public Printable
 {
 public:
 //!Constructor.
@@ -105,14 +110,6 @@ Sets the orientation of the model, using the Euler's angles.
 */
 	void SetRotation (double psi, double theta, double phi);
 /*!
-@param pc the GnomePrintContext used for printing.
-@param width the width of the printed view.
-@param height th height of the printed view.
-
-Prints the current view to a printing context inside a rectangle with predefined size.
-*/
-	void Print (GnomePrintContext *pc, gdouble width, gdouble height);
-/*!
 @param filename the name of the file.
 @param type the type as supported by GdkPixbuf (e.g. "png" or "jpeg").
 @param options the pairs of keys/values to pass GdkPixbuf.
@@ -123,6 +120,17 @@ Export the view contents as an image. The size of the new image is defined by th
 and height parameters.
 */
 	void SaveAsImage (std::string const &filename, char const *type, std::map<std::string, std::string>& options, unsigned width, unsigned height);
+/*!
+@param width the width of the generated image.
+@param height the height of the generated image.
+
+Generates a pixbuf from the current view.
+
+@return the pixbuf containing the generated image
+*/
+	GdkPixbuf *BuildPixbuf (unsigned width, unsigned height);
+	void DoPrint (GtkPrintOperation *print, GtkPrintContext *context);
+	GtkWindow *GetGtkWindow () {return GTK_WINDOW (gtk_widget_get_toplevel (m_pWidget));}
 
 private:
 /*!
@@ -132,16 +140,6 @@ std::
 Called by OnMotion(). x and y are the displacement coordinates of the mouse.
 */
 	void Rotate (gdouble x, gdouble y);
-/*!
-@param width the width of the generated image.
-@param height the height of the generated image.
-
-This method is called by the framework when off-screen renderig fails. This
-will be mich slower than the stahdard method, but mandatory if the video
-card driver does not support off-screen rendering of OpenGL.
-@return the pixbuf containing the generated image
-*/
-	GdkPixbuf *BuildPixbuf (unsigned width, unsigned height);
 
 protected:
 /*!
@@ -154,6 +152,13 @@ private:
 	Matrix m_Euler;
 	double m_Height, m_Width, m_Near, m_Far;
 	double m_Lastx, m_Lasty;
+#ifdef HAVE_GO_CONF_SYNC
+	static GOConfNode *m_ConfNode;
+#else
+	static GConfClient *m_ConfClient;
+#endif
+	static guint m_NotificationId;
+	static int nbViews;
 
 // Properties
 /*!\fn SetAngle(double angle)
