@@ -26,8 +26,12 @@
 #include <string>
 #include <iostream>
 #include <vector>
+#include <cstring>
 
-using namespace gcu;
+using namespace std;
+
+namespace gcu
+{
 
 class TypeDesc
 {
@@ -47,7 +51,8 @@ public:
 static map<string, TypeDesc> Types;
 static vector<string> TypeNames;
 
-Object::Object (TypeId Id)
+Object::Object (TypeId Id):
+	m_Dirty (false)
 {
 	m_Type = Id;
 	m_Id = NULL;
@@ -139,6 +144,8 @@ Object* Object::GetParentOfType (TypeId Id)
 
 void Object::AddChild (Object* object)
 {
+	if (this == object->m_Parent)
+		return;
 	Document* pDoc = GetDocument ();
 	if (!pDoc)
 		cerr << "Cannot add an object outside a document" << endl;
@@ -266,9 +273,10 @@ bool Object::Load (xmlNodePtr node)
 	}
 	child = node->children;
 	while (child) {
-		pObject = CreateObject ((const char*) child->name, this);
+		node = (strcmp ((const char*) child->name, "object"))? child: child->children;
+		pObject = CreateObject ((const char*) node->name, this);
 		if (pObject) {
-			if (!pObject->Load (child))
+			if (!pObject->Load (node))
 				delete pObject;
 		} else {
 			m_Locked--;
@@ -359,10 +367,6 @@ void Object::Add (GtkWidget* w)
 	}
 }
 
-void Object::Print (GnomePrintContext *pc)
-{
-}
-
 void Object::Update (GtkWidget* w)
 {
 	map<string, Object*>::iterator i, end = m_Children.end ();
@@ -597,3 +601,30 @@ void Object::AddMenuCallback (TypeId Id, BuildMenuCb cb)
 	TypeDesc& typedesc = Types[TypeNames[Id]];
 	typedesc.MenuCbs.push_back (cb);
 }
+
+bool Object::SetProperty (unsigned property, char const *value)
+{
+	return true;
+}
+
+
+string Object::GetProperty (unsigned property) const
+{
+	return "";
+}
+
+void Object::OnLoaded ()
+{
+}
+
+void Object::SetDirty (bool dirty)
+{
+	m_Dirty = dirty;
+	if (dirty) {
+		Document *doc = GetDocument ();
+		if (doc)
+			doc->NotifyDirty (this);
+	}
+}
+
+}	//	namespace gcu
