@@ -27,10 +27,9 @@
 #include "document.h"
 #include "view.h"
 #include "window.h"
+#include <gcu/print-setup-dlg.h>
 #include <goffice/gtk/go-action-combo-color.h>
-#include <libgnomeprint/gnome-print-job.h>
-#include <libgnomeprintui/gnome-print-dialog.h>
-#include <libgnomeprintui/gnome-print-job-preview.h>
+#include <goffice/utils/go-image.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
@@ -57,9 +56,19 @@ static void on_file_close (GtkWidget *widget, gc3dWindow* Win)
 	Win->OnFileClose ();
 }
 
+static void on_page_setup (GtkWidget *widget, gc3dWindow* Win)
+{
+	Win->OnPageSetup ();
+}
+
+static void on_print_preview (GtkWidget *widget, gc3dWindow* Win)
+{
+	Win->GetView ()->Print (true);
+}
+
 static void on_file_print (GtkWidget *widget, gc3dWindow* Win)
 {
-	Win->OnFilePrint ();
+	Win->GetView ()->Print (false);
 }
 
 static void on_quit (GtkWidget *widget, gc3dWindow* Win)
@@ -163,6 +172,10 @@ static GtkActionEntry entries[] = {
 		  N_("Open a file"), G_CALLBACK (on_file_open) },
 	  { "SaveAsImage", GTK_STOCK_SAVE_AS, N_("Save As _Image..."), "<control>I",
 		  N_("Save the current file as an image"), G_CALLBACK (on_file_save_as_image) },
+	  { "PageSetup", NULL, N_("Page Set_up..."), NULL,
+		  N_("Setup the page settings for your current printer"), G_CALLBACK (on_page_setup) },
+	  { "PrintPreview", GTK_STOCK_PRINT_PREVIEW, N_("Print Pre_view"), NULL,
+		  N_("Print preview"), G_CALLBACK (on_print_preview) },
 	  { "Print", GTK_STOCK_PRINT, N_("_Print..."), "<control>P",
 		  N_("Print the current scene"), G_CALLBACK (on_file_print) },
 	  { "Close", GTK_STOCK_CLOSE, N_("_Close"), "<control>W",
@@ -193,6 +206,9 @@ static GtkRadioActionEntry radios[] = {
 	{ "Cylinders", "NULL", N_("Cylinders"), NULL,
 		N_("Display a cylinders model"),
 		CYLINDERS },
+	{ "Wireframe", "NULL", N_("Wireframe"), NULL,
+		N_("Display a wireframe model"),
+		WIREFRAME },
 };
 
 static const char *ui_description =
@@ -202,7 +218,10 @@ static const char *ui_description =
 "      <menuitem action='Open'/>"
 "      <menuitem action='SaveAsImage'/>"
 "	   <separator name='file-sep1'/>"
+"      <menuitem action='PageSetup'/>"
+"      <menuitem action='PrintPreview'/>"
 "      <menuitem action='Print'/>"
+"	   <separator name='file-sep2'/>"
 "      <menuitem action='Close'/>"
 "      <menuitem action='Quit'/>"
 "    </menu>"
@@ -210,6 +229,7 @@ static const char *ui_description =
 "      <menuitem action='BallnStick'/>"
 "      <menuitem action='SpaceFill'/>"
 "      <menuitem action='Cylinders'/>"
+"      <menuitem action='Wireframe'/>"
 "	   <separator name='view-sep1'/>"
 "      <menuitem action='Background'/>"
 "    </menu>"
@@ -309,6 +329,20 @@ gc3dWindow::gc3dWindow (gc3dApplication *App, gc3dDocument *Doc)
 	m_View = dynamic_cast<gc3dView *> (m_Doc->GetView ());
 	m_View->SetWindow (this);
 	gtk_container_add (GTK_CONTAINER (vbox), m_View->GetWidget ());
+	switch (Doc->GetDisplay3D ()) {
+	case BALL_AND_STICK:
+		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (gtk_action_group_get_action (action_group, "BallnStick")), true);
+		break;
+	case SPACEFILL:
+		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (gtk_action_group_get_action (action_group, "SpaceFill")), true);
+		break;
+	case CYLINDERS:
+		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (gtk_action_group_get_action (action_group, "Cylinders")), true);
+		break;
+	case WIREFRAME:
+		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (gtk_action_group_get_action (action_group, "Wireframe")), true);
+		break;
+	}
 	gtk_widget_show_all (GTK_WIDGET (m_Window));
 }
 
@@ -328,45 +362,9 @@ void gc3dWindow::OnFileClose ()
 	delete this;
 }
 
-void gc3dWindow::OnFilePrint ()
+void gc3dWindow::OnPageSetup ()
 {
-	GnomePrintConfig* config = gnome_print_config_default ();
-	GnomePrintContext *pc;
-	GnomePrintJob *gpj = gnome_print_job_new (config);
-	int do_preview = 0, copies = 1, collate = 0;
-	GnomePrintDialog *gpd;
-	gpd = GNOME_PRINT_DIALOG (gnome_print_dialog_new (gpj, (const guchar*) "Print test", GNOME_PRINT_DIALOG_COPIES));
-	gnome_print_dialog_set_copies (gpd, copies, collate);
-	switch (gtk_dialog_run (GTK_DIALOG (gpd)))
-	{
-	case GNOME_PRINT_DIALOG_RESPONSE_PREVIEW:
-		do_preview = 1;
-		break;
-	case GNOME_PRINT_DIALOG_RESPONSE_CANCEL:
-		gtk_widget_destroy (GTK_WIDGET (gpd));
-		g_object_unref (gpj);
-		gnome_print_config_unref (config);
-		return;
-	}
-	gtk_widget_destroy (GTK_WIDGET (gpd));
-	pc = gnome_print_job_get_context (gpj);
-	gnome_print_beginpage (pc, (const guchar*)"");
-	gdouble width, height;
-	gnome_print_config_get_double (config, (const guchar*) GNOME_PRINT_KEY_PAPER_WIDTH, &width);
-	gnome_print_config_get_double (config, (const guchar*) GNOME_PRINT_KEY_PAPER_HEIGHT, &height);
-	m_View->Print (pc, width, height);
-	gnome_print_showpage (pc);
-	g_object_unref (pc);
-	gnome_print_job_close (gpj);
-	if (do_preview)
-	{
-		GtkWidget *preview = gnome_print_job_preview_new (gpj, (const guchar*) _("Preview"));
-		gtk_widget_show (preview);
-	} else {
-		gnome_print_job_print (gpj);
-	}
-	g_object_unref (gpj);
-	gnome_print_config_unref (config);
+	new PrintSetupDlg (m_App, m_View);
 }
 
 void gc3dWindow::SetTitle (char const *title)
