@@ -27,6 +27,7 @@
 #include "gchemtable-data-allocator.h"
 #include <gcu/chemistry.h>
 #include <gcu/element.h>
+#include <gcu/print-setup-dlg.h>
 #include <goffice/data/go-data-simple.h>
 #include <goffice/gtk/go-graph-widget.h>
 #include <goffice/gtk/goffice-gtk.h>
@@ -37,21 +38,22 @@
 #include <goffice/graph/gog-label.h>
 #include <goffice/graph/gog-object.h>
 #include <goffice/graph/gog-plot.h>
-#include <goffice/graph/gog-renderer-cairo.h>
 #include <goffice/graph/gog-series.h>
 #include <goffice/graph/gog-style.h>
 #include <goffice/graph/gog-styled-object.h>
+#include <goffice/math/go-math.h>
 #include <goffice/utils/go-locale.h>
 #include <goffice/utils/go-line.h>
 #include <goffice/utils/go-image.h>
 #include <goffice/utils/go-marker.h>
-#include <goffice/utils/go-math.h>
 #include <gsf/gsf-input-memory.h>
 #include <gsf/gsf-output-memory.h>
 #include <glib/gi18n.h>
 #include <map>
+#include <cstring>
 
 using namespace gcu;
+using namespace std;
 
 // FIXME "the following lines should be removed for stable releases"
 #undef PACKAGE
@@ -151,12 +153,12 @@ static void on_copy (GtkWidget *widget, GChemTableCurve *curve)
 
 static void on_print (GtkWidget *widget, GChemTableCurve *curve)
 {
-	curve->OnPrint (false);
+	curve->Print (false);
 }
 
 static void on_print_preview (GtkWidget *widget, GChemTableCurve *curve)
 {
-	curve->OnPrint (true);
+	curve->Print (true);
 }
 
 static void on_page_setup (GtkWidget *widget, GChemTableCurve *curve)
@@ -289,7 +291,8 @@ static const char *ui_web_description =
 "</ui>";
 
 GChemTableCurve::GChemTableCurve (GChemTableApp *App, char const *name):
-	Dialog (App, GLADEDIR"/curve.glade", "curvedlg")
+	Dialog (App, GLADEDIR"/curve.glade", "curvedlg"),
+	Printable ()
 {
 	m_Name = name;
 	m_GraphBox = glade_xml_get_widget (xml, "vbox1");
@@ -508,9 +511,6 @@ GChemTableCurve::GChemTableCurve (GChemTableApp *App, char const *name):
 	gog_dataset_set_dim (GOG_DATASET (label), 0, data, &error);
 	gog_object_add_by_name (obj, "Label", label);
 	m_Graph = go_graph_widget_get_graph (GO_GRAPH_WIDGET (m_GraphWidget));
-	// Initialize print settings
-	m_PageSetup = gtk_page_setup_new ();
-	m_PrintSettings = gtk_print_settings_new ();
 }
 
 GChemTableCurve::~GChemTableCurve ()
@@ -518,44 +518,6 @@ GChemTableCurve::~GChemTableCurve ()
 	curves.erase (m_Name);
 }
 
-static void begin_print (GtkPrintOperation *print, GtkPrintContext *context, gpointer data)
-{
-	gtk_print_operation_set_n_pages (print, 1);
-}
-
-static void draw_page (GtkPrintOperation *print, GtkPrintContext *context, gint page_nr,gpointer data)
-{
-	((GChemTableCurve *) data)->DoPrint (print, context);
-}
-
-void GChemTableCurve::OnPrint (bool preview)
-{
-	GtkPrintOperation *print;
-	GtkPrintOperationResult res;
-
-	print = gtk_print_operation_new ();
-	gtk_print_operation_set_use_full_page (print, false);
-
-    gtk_print_operation_set_print_settings (print, m_PrintSettings);
-    gtk_print_operation_set_default_page_setup (print, m_PageSetup);
-
-	g_signal_connect (print, "begin_print", G_CALLBACK (begin_print), NULL);
-	g_signal_connect (print, "draw_page", G_CALLBACK (draw_page), this);
-	
-	res = gtk_print_operation_run (print,
-								   (preview)? GTK_PRINT_OPERATION_ACTION_PREVIEW:
-								   			GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
-								   GTK_WINDOW (dialog), NULL);
-
-	if (res == GTK_PRINT_OPERATION_RESULT_APPLY) {
-		if (m_PrintSettings != NULL)
-			g_object_unref (m_PrintSettings);
-		m_PrintSettings = GTK_PRINT_SETTINGS (g_object_ref (gtk_print_operation_get_print_settings (print)));
-	}
-
-	g_object_unref (print);
-}
-
 void GChemTableCurve::DoPrint (GtkPrintOperation *print, GtkPrintContext *context)
 {
 	cairo_t *cr;
@@ -564,24 +526,38 @@ void GChemTableCurve::DoPrint (GtkPrintOperation *print, GtkPrintContext *contex
 	cr = gtk_print_context_get_cairo_context (context);
 	width = gtk_print_context_get_width (context);
 	height = gtk_print_context_get_height (context);
-	GogGraph *Graph = GOG_GRAPH (gog_object_dup ((GogObject *) m_Graph, NULL, NULL));
-	gog_graph_set_size (Graph, width, height);
-	GogRenderer *renderer = gog_renderer_new_for_pixbuf (m_Graph);
-	g_object_set (renderer, "cairo", cr, NULL);
-	gog_renderer_update (renderer, width, height, 1.0);
-	g_object_unref (renderer);
-	g_object_unref (Graph);
+
+	int w, h; // size in points
+	w = m_GraphWidget->allocation.width;
+	h = m_GraphWidget->allocation.height;
+	switch (GetScaleType ()) {
+	case GCU_PRINT_SCALE_NONE:
+		break;
+	case GCU_PRINT_SCALE_FIXED:
+		w *= Printable::GetScale ();
+		h *= Printable::GetScale ();
+		break;
+	case GCU_PRINT_SCALE_AUTO:
+		if (GetHorizFit ())
+			w = width;
+		if (GetVertFit ())
+			h = height;
+		break;
+	}
+	double x = 0., y = 0.;
+	if (GetHorizCentered ())
+		x = (width - w) / 2.;
+	if (GetVertCentered ())
+		y = (height - h) / 2.;
+	cairo_save (cr);
+	cairo_translate (cr, x, y);
+	gog_graph_render_to_cairo (m_Graph, cr, w, h);
+	cairo_restore (cr);
 }
 
 void GChemTableCurve::OnPageSetup ()
 {
-	GtkPageSetup *setup = gtk_print_run_page_setup_dialog (
-											dialog,
-											m_PageSetup,
-											m_PrintSettings
-										);
-	g_object_unref (m_PageSetup);
-	m_PageSetup = setup;
+	new PrintSetupDlg (m_App, this);
 }
 
 void GChemTableCurve::OnCopy ()
@@ -615,8 +591,9 @@ void GChemTableCurve::OnProperties ()
 	gct_control_gui_set_owner (tcg, this);
 	GClosure *closure = g_cclosure_new (G_CALLBACK (on_update_graph), tcg,
 					(GClosureNotify) graph_user_config_free_data);
-	gog_guru (m_Graph, GOG_DATA_ALLOCATOR (tcg),
-		       NULL, GTK_WINDOW (dialog), closure);
+	gtk_window_set_transient_for (GTK_WINDOW (dialog), 
+								  GTK_WINDOW (gog_guru (m_Graph, GOG_DATA_ALLOCATOR (tcg),
+		      											NULL, closure)));
 	g_closure_sink (closure);
 }
 
