@@ -24,9 +24,11 @@
 
 #include "config.h"
 #include "gchemtable-app.h"
+#include <libgnomevfs/gnome-vfs-init.h>
 #include <goffice/goffice.h>
 #include <goffice/app/go-plugin.h>
 #include <goffice/app/go-plugin-loader-module.h>
+#include <libintl.h>
 
 using namespace gcu;
 
@@ -34,12 +36,11 @@ int main (int argc, char *argv[])
 {
 	textdomain (GETTEXT_PACKAGE);
 	gtk_init (&argc, &argv);
-	/* Initialize libgoffice */
-	libgoffice_init ();
-	/* Initialize plugins manager */
-	go_plugins_init (NULL, NULL, NULL, NULL, TRUE, GO_PLUGIN_LOADER_MODULE_TYPE);
+	gnome_vfs_init ();
 
 	new GChemTableApp ();
+	/* Initialize plugins manager */
+	go_plugins_init (NULL, NULL, NULL, NULL, TRUE, GO_PLUGIN_LOADER_MODULE_TYPE);
 
 	gtk_main();
 	return 0;
